import numpy as np
from prophet import Prophet


class AnomalyDetector:
    def __init__(self, args=None):
        if args is None:
            args = {}
        self.interval_width = getattr(args, "interval_width", 0.6)
        self.alpha = getattr(args, "alpha", 1.0)
        self.anomaly_threshold = getattr(args, "anomaly_threshold", 0.5)
        self.persistence_minutes = getattr(args, "persistence_minutes", 1)  # Minimum persistence time
        self.input_temporal_granularity = getattr(args, "input_temporal_granularity", 5)

        # Store Prophet-specific parameters
        self.prophet_params = {
            "interval_width": self.interval_width,
            "growth": getattr(args, "growth", "linear"),
            "n_changepoints": getattr(args, "n_changepoints", 25),
            "changepoint_range": getattr(args, "changepoint_range", 0.8),
            "changepoint_prior_scale": getattr(args, "changepoint_prior_scale", 0.05),
            "yearly_seasonality": getattr(args, "yearly_seasonality", "auto"),
            "weekly_seasonality": getattr(args, "weekly_seasonality", "auto"),
            "daily_seasonality": getattr(args, "daily_seasonality", "auto"),
            "seasonality_mode": getattr(args, "seasonality_mode", "additive"),
            "seasonality_prior_scale": getattr(args, "seasonality_prior_scale", 10.0),
            "holidays_prior_scale": getattr(args, "holidays_prior_scale", 10.0),
            "mcmc_samples": getattr(args, "mcmc_samples", 0),
            "uncertainty_samples": getattr(args, "uncertainty_samples", 1000),
        }

    def sigmoid_anomaly_score(self, actual, predicted):
        deviation = actual / predicted
        return 1 / (1 + np.exp(-self.alpha * (deviation - 1)))  # Centered around 1

    def apply_persistence_filter(self, df):
        if self.persistence_minutes <= 0:
            return df

        df = df.sort_values("ds").reset_index(drop=True)  # Ensure proper ordering

        # Convert persistence time to row count
        persistence_window = self.persistence_minutes // self.input_temporal_granularity

        # Track persistent anomaly regions
        df["persistent_anomaly"] = False

        # Find indices of all anomalies
        anomaly_indices = df.index[df["is_anomaly"]].tolist()

        if not anomaly_indices:
            return df

        # Identify contiguous anomaly blocks
        blocks = []
        current_block = [anomaly_indices[0]]

        for i in range(1, len(anomaly_indices)):
            if anomaly_indices[i] - anomaly_indices[i - 1] <= persistence_window:
                current_block.append(anomaly_indices[i])
            else:
                blocks.append(current_block)
                current_block = [anomaly_indices[i]]
        blocks.append(current_block)  # Add last block

        # Mark only the start and end of each anomaly block
        for block in blocks:
            if len(block) >= persistence_window:  # Only consider blocks that meet persistence requirement
                df.loc[block[0], "persistent_anomaly"] = True  # Mark start
                df.loc[block[-1], "persistent_anomaly"] = True  # Mark end

        return df



    def detect_anomalies(self, data, metric="packet_count"):
        """Detects anomalies in time series data using Facebook Prophet."""

        # Prepare data for Prophet
        df = data.rename(columns={"timestamp": "ds", metric: "y"})

        # Fit Prophet model
        model = Prophet(**self.prophet_params)
        model.fit(df)

        # Make future predictions
        future = model.make_future_dataframe(periods=0)  # Predict only for existing data
        forecast = model.predict(future)

        # Merge predictions with original data
        df = df.merge(forecast[["ds", "yhat", "yhat_lower", "yhat_upper"]], on="ds")

        # Identify anomalies (values outside predicted bounds)
        df["is_anomaly"] = (df["y"] > df["yhat_upper"]) | (df["y"] < df["yhat_lower"])

        # Compute anomaly scores
        df["anomaly_score"] = df.apply(
            lambda row: self.sigmoid_anomaly_score(row["y"], row["yhat"])
            if row["is_anomaly"]
            else 0,
            axis=1,
        )

        # Filter anomalies based on score threshold
        df["is_anomaly"] = df["anomaly_score"] > self.anomaly_threshold

        # Apply persistence check
        df = self.apply_persistence_filter(df)

        # Ensure `persistent_anomaly` is a boolean column
        df["persistent_anomaly"] = df["persistent_anomaly"].astype(bool)

        return df  # Return only persistent anomalies
